@@ -2,6 +2,7 @@ require "openid/message"
 require "openid/protocolerror"
 require "openid/kvpost"
 require "openid/consumer/discovery"
+require "openid/urinorm"
 
 module OpenID
   class TypeURIMismatch < ProtocolError
@@ -113,7 +114,7 @@ module OpenID
         when OPENID2_NS
           require_fields = basic_fields + ['op_endpoint']
           require_sigs = basic_sig_fields +
-            ['response_nonce', 'claimed_id', 'assoc_handle',]
+            ['response_nonce', 'claimed_id', 'assoc_handle', 'op_endpoint']
         when OPENID1_NS
           require_fields = basic_fields + ['identity']
           require_sigs = basic_sig_fields
@@ -138,7 +139,7 @@ module OpenID
 
       def verify_return_to
         begin
-          msg_return_to = URI.parse(fetch('return_to'))
+          msg_return_to = URI.parse(URINorm::urinorm(fetch('return_to')))
         rescue URI::InvalidURIError
           raise ProtocolError, ("return_to is not a valid URI")
         end
@@ -188,7 +189,7 @@ module OpenID
 
       def verify_return_to_base(msg_return_to)
         begin
-          app_parsed = URI.parse(@current_url)
+          app_parsed = URI.parse(URINorm::urinorm(@current_url))
         rescue URI::InvalidURIError
           raise ProtocolError, "current_url is not a valid URI: #{@current_url}"
         end
@@ -239,24 +240,17 @@ module OpenID
       end
 
       def create_check_auth_request
-        check_args = {}
+        signed_list = @message.get_arg(OPENID_NS, 'signed', NO_DEFAULT).split(',')
 
-        # Arguments that are always passed to the server and not
-        # included in the signature.
-        for k in ['assoc_handle', 'sig', 'signed', 'invalidate_handle']
-          val = fetch(k, nil)
-          if !val.nil?
-            check_args[k] = val
-          end
-        end
+        # check that we got all the signed arguments
+        signed_list.each {|k|
+          @message.get_aliased_arg(k, NO_DEFAULT)
+        }
 
-        for k in signed_list
-          val = @message.get_aliased_arg(k, NO_DEFAULT)
-          check_args[k] = val
-        end
+        ca_message = @message.copy
+        ca_message.set_arg(OPENID_NS, 'mode', 'check_authentication')
 
-        check_args['mode'] = 'check_authentication'
-        return Message.from_openid_args(check_args)
+        return ca_message
       end
 
       # Process the response message from a check_authentication
@@ -352,7 +346,7 @@ module OpenID
 
         if @endpoint.nil?
           Util.log('No pre-discovered information supplied')
-          discover_and_verify(to_match)
+          discover_and_verify(to_match.claimed_id, [to_match])
         else
           begin
             verify_discovery_single(@endpoint, to_match)
@@ -360,7 +354,7 @@ module OpenID
             Util.log("Error attempting to use stored discovery "\
                      "information: #{why.message}")
             Util.log("Attempting discovery to verify endpoint")
-            discover_and_verify(to_match)
+            discover_and_verify(to_match.claimed_id, [to_match])
           end
         end
 
@@ -376,7 +370,7 @@ module OpenID
 
         if claimed_id.nil?
           if @endpoint.nil?
-            raise StandardError, ("When using OpenID 1, the claimed ID must "\
+            raise ProtocolError, ("When using OpenID 1, the claimed ID must "\
                                   "be supplied, either by passing it through "\
                                   "as a return_to parameter or by using a "\
                                   "session, and supplied to the IdResHandler "\
@@ -414,55 +408,53 @@ module OpenID
         # Either no endpoint was supplied or OpenID 1.x verification
         # of the information that's in the message failed on that
         # endpoint.
-        begin
-          discover_and_verify(to_match)
-        rescue TypeURIMismatch
-          discover_and_verify(to_match_1_0)
-        end
+        discover_and_verify(to_match.claimed_id, [to_match, to_match_1_0])
       end
 
       # Given an endpoint object created from the information in an
       # OpenID response, perform discovery and verify the discovery
       # results, returning the matching endpoint that is the result of
       # doing that discovery.
-      def discover_and_verify(to_match)
-        Util.log("Performing discovery on #{to_match.claimed_id}")
-        _, services = OpenID.discover(to_match.claimed_id)
+      def discover_and_verify(claimed_id, to_match_endpoints)
+        Util.log("Performing discovery on #{claimed_id}")
+        _, services = OpenID.discover(claimed_id)
         if services.length == 0
           # XXX: this might want to be something other than
           # ProtocolError. In Python, it's DiscoveryFailure
           raise ProtocolError, ("No OpenID information found at "\
-                                "#{to_match.claimed_id}")
+                                "#{claimed_id}")
         end
-        verify_discovered_services(services, to_match)
+        verify_discovered_services(claimed_id, services, to_match_endpoints)
       end
 
 
-      def verify_discovered_services(services, to_match)
+      def verify_discovered_services(claimed_id, services, to_match_endpoints)
         # Search the services resulting from discovery to find one
         # that matches the information from the assertion
         failure_messages = []
         for endpoint in services
-          begin
-            verify_discovery_single(endpoint, to_match)
-          rescue ProtocolError => why
-            failure_messages << why.message
-          else
-            # It matches, so discover verification has
-            # succeeded. Return this endpoint.
-            @endpoint = endpoint
-            return
+          for to_match_endpoint in to_match_endpoints
+            begin
+              verify_discovery_single(endpoint, to_match_endpoint)
+            rescue ProtocolError => why
+              failure_messages << why.message
+            else
+              # It matches, so discover verification has
+              # succeeded. Return this endpoint.
+              @endpoint = endpoint
+              return
+            end
           end
         end
 
-        Util.log("Discovery verification failure for #{to_match.claimed_id}")
+        Util.log("Discovery verification failure for #{claimed_id}")
         failure_messages.each do |failure_message|
           Util.log(" * Endpoint mismatch: " + failure_message)
         end
 
         # XXX: is DiscoveryFailure in Python OpenID
         raise ProtocolError, ("No matching endpoint found after "\
-                              "discovering #{to_match.claimed_id}")
+                              "discovering #{claimed_id}")
       end
 
       def verify_discovery_single(endpoint, to_match)
@@ -477,14 +469,14 @@ module OpenID
         # Fragments do not influence discovery, so we can't compare a
         # claimed identifier with a fragment to discovered information.
         defragged_claimed_id =
-          case Yadis::XRI.identifier_scheme(endpoint.claimed_id)
+          case Yadis::XRI.identifier_scheme(to_match.claimed_id)
           when :xri
-            endpoint.claimed_id
+            to_match.claimed_id
           when :uri
             begin
-              parsed = URI.parse(endpoint.claimed_id)
+              parsed = URI.parse(to_match.claimed_id)
             rescue URI::InvalidURIError
-              endpoint.claimed_id
+              to_match.claimed_id
             else
               parsed.fragment = nil
               parsed.to_s
