@@ -4,15 +4,11 @@ require "logger"
 
 require "openid/extras"
 
-srand(Time.now.to_f)
-
 # See OpenID::Consumer or OpenID::Server modules, as well as the store classes
 module OpenID
   class AssertionError < Exception
   end
 
-  VERSION = "2.0.4"
-
   # Exceptions that are raised by the library are subclasses of this
   # exception type, so if you want to catch all exceptions raised by
   # the library, you can catch OpenIDError
@@ -90,6 +86,25 @@ module OpenID
     def Util.log(message)
       logger.info(message)
     end
+
+    def Util.auto_submit_html(form, title='OpenID transaction in progress')
+      return "
+<html>
+<head>
+  <title>#{title}</title>
+</head>
+<body onload='document.forms[0].submit();'>
+#{form}
+<script>
+var elements = document.forms[0].elements;
+for (var i = 0; i < elements.length; i++) {
+  elements[i].style.display = \"none\";
+}
+</script>
+</body>
+</html>
+"
+    end
   end
 
 end
