@@ -27,7 +27,8 @@ module OpenID
 
         OPENID1_SIGNED = ['return_to', 'identity']
         OPENID2_SIGNED =
-          OPENID1_SIGNED + ['response_nonce', 'claimed_id', 'assoc_handle']
+          OPENID1_SIGNED + ['response_nonce', 'claimed_id', 'assoc_handle',
+                            'op_endpoint']
 
         def mkMsg(ns, fields, signed_fields)
           msg = Message.new(ns)
@@ -99,6 +100,35 @@ module OpenID
           end
         end
 
+        def test_112
+          args = {'openid.assoc_handle' => 'fa1f5ff0-cde4-11dc-a183-3714bfd55ca8', 
+                  'openid.claimed_id' => 'http://binkley.lan/user/test01', 
+                  'openid.identity' => 'http://test01.binkley.lan/', 
+                  'openid.mode' => 'id_res', 
+                  'openid.ns' => 'http://specs.openid.net/auth/2.0', 
+                  'openid.ns.pape' => 'http://specs.openid.net/extensions/pape/1.0', 
+                  'openid.op_endpoint' => 'http://binkley.lan/server', 
+                  'openid.pape.auth_policies' => 'none', 
+                  'openid.pape.auth_time' => '2008-01-28T20:42:36Z', 
+                  'openid.pape.nist_auth_level' => '0', 
+                  'openid.response_nonce' => '2008-01-28T21:07:04Z99Q=', 
+                  'openid.return_to' => 'http://binkley.lan:8001/process?janrain_nonce=2008-01-28T21%3A07%3A02Z0tMIKx', 
+                  'openid.sig' => 'YJlWH4U6SroB1HoPkmEKx9AyGGg=', 
+                  'openid.signed' => 'assoc_handle,identity,response_nonce,return_to,claimed_id,op_endpoint,pape.auth_time,ns.pape,pape.nist_auth_level,pape.auth_policies' 
+	         } 
+          assert_equal(args['openid.ns'], OPENID2_NS)
+          incoming = Message.from_post_args(args)
+          assert(incoming.is_openid2)
+          idres = IdResHandler.new(incoming, nil)
+          car = idres.send(:create_check_auth_request)
+          expected_args = args.dup
+          expected_args['openid.mode'] = 'check_authentication'
+          expected = Message.from_post_args(expected_args)
+          assert(expected.is_openid2)
+          assert_equal(expected, car)
+          assert_equal(expected_args, car.to_post_args)
+        end        
+
         def test_no_signed_list
           msg = Message.new(OPENID2_NS)
           idres = IdResHandler.new(msg, nil)
@@ -201,6 +231,11 @@ module OpenID
           [ [base, {}],
             [base + "?another=arg", {'another' => 'arg'}],
             [base + "?another=arg#frag", {'another' => 'arg'}],
+            ['HTTP'+base[4..-1], {}],
+            [base.sub('com', 'COM'), {}],
+            ['http://example.janrain.com:80/path', {}],
+            ['http://example.janrain.com/p%61th', {}],
+            ['http://example.janrain.com/./path',{}],
           ].each do |return_to, args|
             args['openid.return_to'] = return_to
             msg = Message.from_post_args(args)
@@ -370,20 +405,12 @@ module OpenID
         end
 
         def test_create_check_auth_request_success
-          msg = call_idres_method(:create_check_auth_request) {}
-          openid_args = @message.get_args(OPENID_NS)
-          openid_args['mode'] = 'check_authentication'
-          assert_equal(openid_args, msg.to_args)
+          ca_msg = call_idres_method(:create_check_auth_request) {}
+          expected = @message.copy
+          expected.set_arg(OPENID_NS, 'mode', 'check_authentication')
+          assert_equal(expected, ca_msg)
         end
 
-        def test_create_check_auth_request_success_extra
-          @message.set_arg(OPENID_NS, 'cookies', 'chocolate_chip')
-          msg = call_idres_method(:create_check_auth_request) {}
-          openid_args = @message.get_args(OPENID_NS)
-          openid_args['mode'] = 'check_authentication'
-          openid_args.delete('cookies')
-          assert_equal(openid_args, msg.to_args)
-        end
       end
 
       class CheckAuthResponseTest < Test::Unit::TestCase
@@ -554,11 +581,45 @@ module OpenID
 
         def test_openid1_no_endpoint
           @endpoint = nil
-          assert_raises(StandardError) {
+          assert_raises(ProtocolError) {
             call_verify({'identity' => 'snakes on a plane'})
           }
         end
 
+        def test_openid1_fallback_1_0
+          claimed_id = 'http://claimed.id/'
+          @endpoint = nil
+          resp_mesg = Message.from_openid_args({
+            'ns' => OPENID1_NS,
+            'identity' => claimed_id,
+            })
+
+          # Pass the OpenID 1 claimed_id this way since we're passing
+          # None for the endpoint.
+          resp_mesg.set_arg(BARE_NS, 'openid1_claimed_id', claimed_id)
+
+          # We expect the OpenID 1 discovery verification to try
+          # matching the discovered endpoint against the 1.1 type and
+          # fall back to 1.0.
+          expected_endpoint = OpenIDServiceEndpoint.new
+          expected_endpoint.type_uris = [OPENID_1_0_TYPE]
+          expected_endpoint.local_id = nil
+          expected_endpoint.claimed_id = claimed_id
+  
+          hacked_discover = Proc.new {
+            |_claimed_id| ['unused', [expected_endpoint]]
+          }
+          idres = IdResHandler.new(resp_mesg, nil, nil, @endpoint)
+          assert_log_matches('Performing discovery') {
+            OpenID.with_method_overridden(:discover, hacked_discover) {
+              idres.send(:verify_discovery_results)
+            }
+          }
+          actual_endpoint = idres.instance_variable_get(:@endpoint)
+          assert_equal(actual_endpoint, expected_endpoint)
+
+        end
+
         def test_openid2_no_op_endpoint
           assert_protocol_error("Missing required field: "\
                                 "<#{OPENID2_NS}>op_endpoint") {
@@ -603,12 +664,12 @@ module OpenID
                                  'identity' => 'sour grapes',
                                  'claimed_id' => 'monkeysoft',
                                  'op_endpoint' => 'Phone Home'}) do |idres|
-              idres.instance_def(:discover_and_verify) do |to_match|
+              idres.instance_def(:discover_and_verify) do |claimed_id, endpoints|
                 @endpoint = endpoint
               end
             end
           }
-          assert(endpoint.equal?(result))
+          assert_equal(endpoint, result)
         end
 
 
@@ -625,8 +686,7 @@ module OpenID
                                  'identity' => 'sour grapes',
                                  'claimed_id' => 'monkeysoft',
                                  'op_endpoint' => 'Green Cheese'}) do |idres|
-                        idres.extend(InstanceDefExtension)
-              idres.instance_def(:discover_and_verify) do |to_match|
+              idres.instance_def(:discover_and_verify) do |claimed_id, endpoints|
                 @endpoint = endpoint
               end
             end
@@ -634,6 +694,22 @@ module OpenID
           assert(endpoint.equal?(result))
         end
 
+        def test_verify_discovery_single_claimed_id_mismatch
+          idres = IdResHandler.new(nil, nil)
+          @endpoint.local_id = 'my identity'
+          @endpoint.claimed_id = 'http://i-am-sam/'
+          @endpoint.server_url = 'Phone Home'
+          @endpoint.type_uris = [OPENID_2_0_TYPE]
+
+          to_match = @endpoint.dup
+          to_match.claimed_id = 'http://something.else/'
+
+          e = assert_raises(ProtocolError) {
+            idres.send(:verify_discovery_single, @endpoint, to_match)
+          }
+          assert(e.to_s =~ /different subjects/)
+        end
+
         def test_openid2_use_pre_discovered
           @endpoint.local_id = 'my identity'
           @endpoint.claimed_id = 'http://i-am-sam/'
@@ -669,8 +745,9 @@ module OpenID
 
           idres = IdResHandler.new(msg, nil, nil, @endpoint)
           idres.extend(InstanceDefExtension)
-          idres.instance_def(:discover_and_verify) { |to_match|
-            me.assert_equal(endpoint.claimed_id, to_match.claimed_id)
+          idres.instance_def(:discover_and_verify) { |claimed_id, to_match|
+            me.assert_equal(endpoint.claimed_id, to_match[0].claimed_id)
+            me.assert_equal(claimed_id, endpoint.claimed_id)
             raise ProtocolError, text
           }
           assert_log_matches('Error attempting to use stored',
@@ -709,7 +786,7 @@ module OpenID
             assert_raises(verified_error) {
               call_verify_modify({'ns' => OPENID1_NS,
                                    'identity' => @endpoint.local_id}) { |idres|
-                idres.instance_def(:discover_and_verify) do |to_match|
+                idres.instance_def(:discover_and_verify) do |claimed_id, endpoints|
                   raise verified_error
                 end
               }
@@ -817,7 +894,7 @@ module OpenID
           assert_log_matches('Performing discovery on') do
             assert_protocol_error('No OpenID information found') do
               OpenID.with_method_overridden(:discover, disco) do
-                idres.send(:discover_and_verify, endpoint)
+                idres.send(:discover_and_verify, :sentinel, [endpoint])
               end
             end
           end
@@ -834,7 +911,8 @@ module OpenID
           idres = IdResHandler.new(nil, nil)
           assert_log_matches('Discovery verification failure') do
             assert_protocol_error('No matching endpoint') do
-              idres.send(:verify_discovered_services, [], endpoint)
+              idres.send(:verify_discovered_services,
+                         'http://bogus.id/', [], [endpoint])
             end
           end
         end
